/*
 * irreco - Ir Remote Control
 * Copyright (C) 2007  Arto Karppinen (arto.karppinen@iki.fi)
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#include "irreco_background_dlg.h"
#include <hildon/hildon-color-button.h>
#include <hildon/hildon-file-chooser-dialog.h>


/**
 * @typedef IrrecoBackgroundDlg
 *
 * Allow user to setup layout background.
 */
 
#define IRRECO_BACKGROUND_PREVIEW_WIDHT		(IRRECO_SCREEN_WIDTH / 4)
#define IRRECO_BACKGROUND_PREVIEW_HEIGHT	(IRRECO_SCREEN_HEIGHT / 4)



/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/
/* Prototypes                                                                 */
/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/

void irreco_background_dlg_sync_radio(IrrecoBackgroundDlg * background_dlg);
gboolean 
irreco_background_dlg_draw_preview(IrrecoBackgroundDlg * background_dlg);
gboolean 
irreco_background_dlg_draw_preview_custom(IrrecoBackgroundDlg * background_dlg,
					  const gchar * image, 
					  const GdkColor * bg_color);
void irreco_background_dlg_color_clicked(GtkColorButton *widget,
					 IrrecoBackgroundDlg * background_dlg);
void irreco_background_dlg_type_toggled(GtkToggleButton * togglebutton,
					IrrecoBackgroundDlg * background_dlg);
void irreco_background_dlg_image_clicked(gpointer * ignore,
					 IrrecoBackgroundDlg * background_dlg);
void irreco_background_dlg_preview_clicked(gpointer * ignore1, 
					   gpointer * ignore2, 
					  IrrecoBackgroundDlg * background_dlg);



/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/
/* Public functions                                                          */
/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/

IrrecoBackgroundDlg *irreco_background_dlg_create(GtkWindow *parent_window)
{
	IrrecoBackgroundDlg *background_dlg;
	GtkWidget *vbox;
	GtkWidget *hbox;
	GtkWidget *vbox_left;
	GtkWidget *vbox_right;
	GtkTable *table_right;
	
	GtkWidget *label_left;
	GtkWidget *label_right;
	GtkWidget *label_color;
	GtkWidget *label_image;
	
	GtkWidget *radio_default_a;
	GtkWidget *radio_color_a;
	GtkWidget *radio_image_a;
	
	GtkWidget *button_image;
	GtkWidget *preview_frame;
	GtkWidget *preview_event_box;
	IRRECO_ENTER
	
	background_dlg = g_slice_new0(IrrecoBackgroundDlg);
	background_dlg->filename = g_string_new(NULL);
	background_dlg->dialog = gtk_dialog_new_with_buttons(
		_("Background"), parent_window,
		GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT | 
		GTK_DIALOG_NO_SEPARATOR, 
		GTK_STOCK_CANCEL, GTK_RESPONSE_REJECT,
		GTK_STOCK_OK, GTK_RESPONSE_ACCEPT,
		NULL);
		
	/* Create widgets. */
	vbox = GTK_DIALOG(background_dlg->dialog)->vbox;
	hbox = gtk_hbox_new(0, 12);
	vbox_left = gtk_vbox_new(0, 0);
	vbox_right = gtk_vbox_new(0, 0);
	
	label_left = irreco_gtk_label_bold(_("Background"), 0, 0, 0, 6, 0, 0);
	label_right = irreco_gtk_label_bold(_("Select"), 0, 0, 0, 6, 0, 0);
	label_color = irreco_gtk_label(_("Color"), 0, 0.5, 0, 0, 12, 0);
	label_image = irreco_gtk_label(_("Image"), 0, 0.5, 0, 0, 12, 0);
	
	background_dlg->radio_default = 
		gtk_radio_button_new_with_label(
			NULL, _("Default image"));
	background_dlg->radio_color = 
		gtk_radio_button_new_with_label_from_widget(
			GTK_RADIO_BUTTON(background_dlg->radio_default), 
			_("Color only"));
	background_dlg->radio_image = 
		gtk_radio_button_new_with_label_from_widget(
			GTK_RADIO_BUTTON(background_dlg->radio_color), 
			_("Color and Image"));
	radio_default_a = irreco_gtk_pad(background_dlg->radio_default, 
					 0, 0, 12, 0);
	radio_color_a = irreco_gtk_pad(background_dlg->radio_color, 
				       0, 0, 12, 0);
	radio_image_a = irreco_gtk_pad(background_dlg->radio_image,
				       0, 0, 12, 0);
	
	background_dlg->color_button = hildon_color_button_new();
	button_image = gtk_button_new_with_label(_("Select"));
	
	preview_frame = gtk_frame_new(NULL);
	preview_event_box = gtk_event_box_new();
	background_dlg->preview = gtk_drawing_area_new();
	gtk_drawing_area_size(GTK_DRAWING_AREA(background_dlg->preview),
			      IRRECO_BACKGROUND_PREVIEW_WIDHT,
			      IRRECO_BACKGROUND_PREVIEW_HEIGHT);
				
	/* Build dialog. */
	table_right = GTK_TABLE(gtk_table_new(2, 2, FALSE));
	gtk_table_set_row_spacings(table_right, 6);
	gtk_table_set_col_spacings(table_right, 12);
	gtk_table_attach_defaults(table_right, label_color, 0, 1, 0, 1);
	gtk_table_attach_defaults(table_right, label_image, 0, 1, 1, 2);
	gtk_table_attach_defaults(table_right, background_dlg->color_button,
				  1, 2, 0, 1);
	gtk_table_attach_defaults(table_right, button_image, 1, 2, 1, 2);
	
	gtk_box_pack_start_defaults(GTK_BOX(vbox), 
				    irreco_gtk_pad(hbox, 12, 0, 12, 12));
	gtk_box_pack_start_defaults(GTK_BOX(hbox), vbox_left);
	gtk_box_pack_start_defaults(GTK_BOX(hbox), vbox_right);
	gtk_box_pack_start(GTK_BOX(vbox_left), label_left, 0, 0, 0);
	gtk_box_pack_start(GTK_BOX(vbox_left), radio_default_a, 0, 0, 0);
	gtk_box_pack_start(GTK_BOX(vbox_left), radio_color_a, 0, 0, 0);
	gtk_box_pack_start(GTK_BOX(vbox_left), radio_image_a, 0, 0, 0);
	gtk_box_pack_start(GTK_BOX(vbox_right), label_right, 0, 0, 0);
	gtk_box_pack_start(GTK_BOX(vbox_right), GTK_WIDGET(table_right), 
			   0, 0, 0);
	
	gtk_container_add(GTK_CONTAINER(preview_event_box), preview_frame);
	gtk_container_add(GTK_CONTAINER(preview_frame),background_dlg->preview);
	gtk_box_pack_start_defaults(GTK_BOX(vbox), irreco_gtk_align(
				    preview_event_box, 
				    0.5, 0.5, 0, 0, 12, 0, 0, 0));

	/* Radio button signals. */
	g_signal_connect(G_OBJECT(background_dlg->radio_default), "toggled",
			 G_CALLBACK(irreco_background_dlg_type_toggled), 
			 background_dlg);
	g_object_set_data(G_OBJECT(background_dlg->radio_default),  
			  "IrrecoButtonLayoutBgType",
			  (gpointer) IRRECO_BACKGROUND_DEFAULT);
	g_signal_connect(G_OBJECT(background_dlg->radio_color), "toggled",
			 G_CALLBACK(irreco_background_dlg_type_toggled), 
			 background_dlg);
	g_object_set_data(G_OBJECT(background_dlg->radio_color),  
			  "IrrecoButtonLayoutBgType",
			  (gpointer) IRRECO_BACKGROUND_COLOR);
	g_signal_connect(G_OBJECT(background_dlg->radio_image), "toggled",
			 G_CALLBACK(irreco_background_dlg_type_toggled), 
			 background_dlg);
	g_object_set_data(G_OBJECT(background_dlg->radio_image),  
			  "IrrecoButtonLayoutBgType",
			  (gpointer) IRRECO_BACKGROUND_IMAGE);
	
	/* Button signals. */
	g_signal_connect(G_OBJECT(button_image), "clicked",
			 G_CALLBACK(irreco_background_dlg_image_clicked), 
			 background_dlg);
	g_signal_connect(G_OBJECT(background_dlg->color_button), "clicked",
			 G_CALLBACK(irreco_background_dlg_color_clicked), 
			 background_dlg);
	g_signal_connect(G_OBJECT(preview_event_box), "button-release-event",
			 G_CALLBACK(irreco_background_dlg_preview_clicked), 
			 background_dlg);
	
	gtk_widget_show_all(background_dlg->dialog);
	IRRECO_RETURN_PTR(background_dlg);
}

/*
 * Destroys and hides dialog.
 */
void irreco_background_dlg_destroy(IrrecoBackgroundDlg * background_dlg)
{
	IRRECO_ENTER
	gtk_widget_destroy(background_dlg->dialog);
	g_string_free(background_dlg->filename, TRUE);
	g_slice_free(IrrecoBackgroundDlg, background_dlg);
	IRRECO_RETURN
}

/*
 * Show dialog, and ask user for input.
 */
gboolean irreco_background_dlg_run(IrrecoBackgroundDlg * background_dlg,
				   IrrecoButtonLayout * irreco_layout)
{
	IRRECO_ENTER
	
	/* Setup dialog. */
	background_dlg->type = irreco_button_layout_get_bg_type(irreco_layout);
	irreco_gstring_set(background_dlg->filename, 
			   irreco_button_layout_get_bg_image(irreco_layout));
	hildon_color_button_set_color(
		HILDON_COLOR_BUTTON(background_dlg->color_button),
		irreco_button_layout_get_bg_color(irreco_layout));
		
	/* Prepare ui. */
	irreco_background_dlg_sync_radio(background_dlg);
	irreco_background_dlg_draw_preview(background_dlg);
					   
	if (gtk_dialog_run(GTK_DIALOG(background_dlg->dialog))
	    == GTK_RESPONSE_ACCEPT) {
		GdkColor *color;
		g_object_get(G_OBJECT(background_dlg->color_button), 
			     "color", &color, NULL);
		irreco_button_layout_set_bg_type(irreco_layout, 
						 background_dlg->type);
		irreco_button_layout_set_bg_color(irreco_layout, color);   
		irreco_button_layout_set_bg_image(
			irreco_layout, background_dlg->filename->str);
			
		IRRECO_PRINTF("%i\n", 
			irreco_button_layout_get_bg_type(irreco_layout));
			
		IRRECO_RETURN_BOOL(TRUE);
	}
	IRRECO_RETURN_BOOL(FALSE);
}



/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/
/* Private functions                                                          */
/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/

void irreco_background_dlg_sync_radio(IrrecoBackgroundDlg * background_dlg)
{
	IRRECO_ENTER
	switch (background_dlg->type) {
	case IRRECO_BACKGROUND_DEFAULT:
		gtk_widget_grab_focus(background_dlg->radio_default);
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(
					     background_dlg->radio_default),
					     TRUE);
		break;	
	
	case IRRECO_BACKGROUND_COLOR:
		gtk_widget_grab_focus(background_dlg->radio_color);
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(
					     background_dlg->radio_color),
					     TRUE);
		break;
		
	case IRRECO_BACKGROUND_IMAGE:
		gtk_widget_grab_focus(background_dlg->radio_image);
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(
					     background_dlg->radio_image),
					     TRUE);
		break;
	}
	IRRECO_RETURN
}

/*
 * Draw preview with current image.
 */
gboolean 
irreco_background_dlg_draw_preview(IrrecoBackgroundDlg * background_dlg)
{
	const gchar *image = NULL;
	const GdkColor *color = NULL;
	IRRECO_ENTER
	
	switch (background_dlg->type) {
	case IRRECO_BACKGROUND_DEFAULT:
		image = irreco_button_layout_default_image;
		color = &irreco_button_layout_default_color;
		break;
		
	case IRRECO_BACKGROUND_IMAGE:
		image = background_dlg->filename->str;
		if (irreco_str_isempty(image)) image = NULL;
	
	case IRRECO_BACKGROUND_COLOR:
		g_object_get(G_OBJECT(background_dlg->color_button), 
			     "color", &color, NULL);
		break;
	}
	
	IRRECO_RETURN_BOOL(irreco_background_dlg_draw_preview_custom(
		background_dlg, image, color));
}

/*
 * Draw preview with custom image.
 */
gboolean 
irreco_background_dlg_draw_preview_custom(IrrecoBackgroundDlg * background_dlg,
					  const gchar * image, 
					  const GdkColor * bg_color)
{
	GError *error = NULL;
	GdkPixbuf *pixbuf = NULL;
	GdkPixmap *pixmap = NULL;
	GdkGC *bg_gc = NULL;
	IRRECO_ENTER
	
	g_assert(background_dlg != NULL);
	
	/* Attempt to load the image. */
	if (image != NULL) {
		pixbuf = gdk_pixbuf_new_from_file_at_scale(image, 
			IRRECO_BACKGROUND_PREVIEW_WIDHT,
			IRRECO_BACKGROUND_PREVIEW_HEIGHT,
			FALSE, &error);
		if (irreco_gerror_check_print(&error)) {
			IRRECO_RETURN_BOOL(FALSE);
		}
	}
	
	pixmap = gdk_pixmap_new(GDK_DRAWABLE(
				background_dlg->preview->window), 
				IRRECO_BACKGROUND_PREVIEW_WIDHT, 
				IRRECO_BACKGROUND_PREVIEW_HEIGHT,
				-1);
				
	/* Fill background with solid color. */
	bg_gc = gdk_gc_new(GDK_DRAWABLE(background_dlg->preview->window));
	gdk_gc_set_rgb_fg_color(bg_gc, bg_color);
	gdk_gc_set_rgb_bg_color(bg_gc, bg_color);
	gdk_draw_rectangle(GDK_DRAWABLE(pixmap), bg_gc, TRUE, 0, 0, 
			   IRRECO_BACKGROUND_PREVIEW_WIDHT, 
			   IRRECO_BACKGROUND_PREVIEW_HEIGHT);
	g_object_unref(G_OBJECT(bg_gc));
	
	/* Draw images to pixmap. */
	if (pixbuf != NULL) {
		gdk_draw_pixbuf(GDK_DRAWABLE(pixmap), NULL, pixbuf, 0, 0, 0, 0,
				IRRECO_BACKGROUND_PREVIEW_WIDHT,
				IRRECO_BACKGROUND_PREVIEW_HEIGHT,
				GDK_RGB_DITHER_NORMAL, 0, 0);
	}
	
	/* Set background image, and queque redraw, so the image is shown.*/	
	gdk_window_set_back_pixmap(background_dlg->preview->window,
				   pixmap, FALSE);
	gtk_widget_queue_draw_area(background_dlg->preview, 0, 0,
				background_dlg->preview->allocation.width,
				background_dlg->preview->allocation.height);
	
	if (pixbuf != NULL) g_object_unref(G_OBJECT(pixbuf));
	if (pixmap != NULL) g_object_unref(G_OBJECT(pixmap));
	IRRECO_RETURN_BOOL(TRUE);
}

void irreco_background_dlg_select_image(IrrecoBackgroundDlg * background_dlg)
{
	GdkColor *color = NULL;
	GtkWidget *file_dlg = NULL;
	gchar *image_dir = NULL;
	IRRECO_ENTER
	
	/* Create image select dialog. */
	file_dlg = hildon_file_chooser_dialog_new(GTK_WINDOW(
						  background_dlg->dialog), 
						  GTK_FILE_CHOOSER_ACTION_OPEN);
	gtk_window_set_title(GTK_WINDOW(file_dlg),_("Select background image"));
	gtk_file_chooser_set_local_only(GTK_FILE_CHOOSER(file_dlg), TRUE);
	image_dir = g_build_path("/", getenv("HOME"), "MyDocs/.images/", NULL);
	gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER(file_dlg), 
					    image_dir);
	g_free(image_dir);
	
	/* Loop until user cancels or we get a valid image. */
	gtk_widget_show_all(GTK_WIDGET(file_dlg));
	while (gtk_dialog_run(GTK_DIALOG(file_dlg)) == GTK_RESPONSE_OK) {
		gchar *filename = gtk_file_chooser_get_filename(
			GTK_FILE_CHOOSER(file_dlg));
		IRRECO_PRINTF("Attempting to display background "
			      "image: \"%s\".\n", filename);
					
		/* Attempt to display the image. */
		g_object_get(G_OBJECT(background_dlg->color_button), 
			     "color", &color, NULL);
		if (irreco_background_dlg_draw_preview_custom(
				background_dlg, filename, color)) {
			IRRECO_PRINTF("Image OK.\n");
			irreco_gstring_set_and_free(
				background_dlg->filename, filename);
			filename = NULL;
					   
			/* Set type as image. */
			if (background_dlg->type != IRRECO_BACKGROUND_IMAGE) {
				gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(
					background_dlg->radio_image), TRUE);	
			}
			break;
			
		/* Cleanup */
		} else {
			gchar *basename = g_path_get_basename(filename);
			irreco_error_dlg_printf(GTK_WINDOW(file_dlg), 
						_("Cannot open image \"%s\""),
						basename);
			IRRECO_PRINTF("Image invalid.\n");
			g_free(basename);
			g_free(filename);
		}
		
	}
	
	gtk_widget_grab_focus(background_dlg->radio_image);
	gtk_widget_destroy(file_dlg);
	IRRECO_RETURN
}



/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/
/* Signal handlers.                                                           */
/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/

void irreco_background_dlg_color_clicked(GtkColorButton *widget,
					   IrrecoBackgroundDlg * background_dlg)
{
	IRRECO_ENTER
	
	/* Make sure the background type is color or image. */
	if (background_dlg->type == IRRECO_BACKGROUND_DEFAULT) {
		/* This should trigger a redraw ... */
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(
					     background_dlg->radio_color),
					     TRUE);
	} else {
		irreco_background_dlg_draw_preview(background_dlg);
	}
	IRRECO_RETURN
}

void irreco_background_dlg_type_toggled(GtkToggleButton * togglebutton,
					 IrrecoBackgroundDlg * background_dlg)
{
	IrrecoButtonLayoutBgType type;
	IRRECO_ENTER
	
	if (gtk_toggle_button_get_active(togglebutton) == TRUE) {
		type = (IrrecoButtonLayoutBgType) g_object_get_data(
			G_OBJECT(togglebutton), "IrrecoButtonLayoutBgType");
		background_dlg->type = type;
		IRRECO_PRINTF("Type: %i\n", background_dlg->type);
	}
	
	irreco_background_dlg_draw_preview(background_dlg);
	IRRECO_RETURN
}

void irreco_background_dlg_image_clicked(gpointer * ignore,
					 IrrecoBackgroundDlg * background_dlg)
{
	IRRECO_ENTER
	irreco_background_dlg_select_image(background_dlg);
	IRRECO_RETURN
}

void irreco_background_dlg_preview_clicked(gpointer * ignore1, 
					   gpointer * ignore2, 
					   IrrecoBackgroundDlg * background_dlg)
{
	IRRECO_ENTER
	irreco_background_dlg_select_image(background_dlg);
	IRRECO_RETURN
}




