/*
 * irreco - Ir Remote Control
 * Copyright (C) 2007  Arto Karppinen (arto.karppinen@iki.fi)
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */


/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/
/* Typedef                                                                    */
/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/
/*
 * Make sure that typedefs are available before we include anything elese.
 *
 * This makes sure that whatever other structures that depend on structures 
 * defined in this file will compile OK recardles of header inclusion order.
 */
#ifndef __IRRECO_CMD_CHAIN_H_TYPEDEF__
#define __IRRECO_CMD_CHAIN_H_TYPEDEF__
typedef struct _IrrecoCmdChain IrrecoCmdChain;
#endif /* __IRRECO_CMD_CHAIN_H_TYPEDEF__ */
 


/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/
/* Include                                                                    */
/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/
#ifndef __IRRECO_CMD_CHAIN_H__
#define __IRRECO_CMD_CHAIN_H__
#include "irreco.h"
#include "irreco_cmd.h"



/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/
/* Datatypes                                                                  */
/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/
struct _IrrecoCmdChain {
	GList *command_list;
	glong execution_rate;
};



/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/
/* Macro                                                                      */
/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/
#define IRRECO_CMD_CHAIN_FOREACH(_cmd_chain, _cmd_var_name)	       	       \
	{ GList *_list;							       \
	IrrecoCmd *_cmd_var_name;					       \
	if ((_list = g_list_first(_cmd_chain->command_list)) != NULL) do {     \
		_cmd_var_name = (IrrecoCmd *) _list->data; {
#define IRRECO_CMD_CHAIN_FOREACH_END					       \
	}} while ((_list = g_list_next(_list)) != NULL); }



/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/
/* Prototypes                                                                 */
/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/
IrrecoCmdChain *irreco_cmd_chain_create();
void irreco_cmd_chain_remove_all(IrrecoCmdChain * irreco_cmd_chain);
void irreco_cmd_chain_destroy(IrrecoCmdChain * irreco_cmd_chain);
void irreco_cmd_chain_set_execution_rate(IrrecoCmdChain * irreco_cmd_chain, 
					 glong execution_rate);
void irreco_cmd_chain_append(IrrecoCmdChain * irreco_cmd_chain, 
			     IrrecoCmd * irreco_cmd);
void irreco_cmd_chain_append_copy(IrrecoCmdChain * irreco_cmd_chain, 
				  IrrecoCmd * irreco_cmd);
void irreco_cmd_chain_remove(IrrecoCmdChain * irreco_cmd_chain, guint index);
void irreco_cmd_chain_move(IrrecoCmdChain * irreco_cmd_chain, 
			   guint from_index, guint to_index);
void irreco_cmd_chain_copy(IrrecoCmdChain * from, 
			   IrrecoCmdChain * to);
guint irreco_cmd_chain_length(IrrecoCmdChain * irreco_cmd_chain);
void irreco_cmd_chain_print(IrrecoCmdChain * irreco_cmd_chain);
gboolean irreco_cmd_chain_execute(IrrecoCmdChain * irreco_cmd_chain,
				  IrrecoData * irreco_data,
				  GFunc call_when_done,
				  gpointer user_data);

#endif /* __IRRECO_CMD_CHAIN_H__ */


