/*
 * irreco - Ir Remote Control
 * Copyright (C) 2007  Arto Karppinen (arto.karppinen@iki.fi)
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#include "irreco_cmd_chain_dlg.h"

/**
 * @typedef IrrecoCmdChainDlg
 *
 * Dialog which shows IrrecoCmdChainEditor.
 */


 
/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/
/* Prototypes                                                                 */
/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/
void irreco_cmd_chain_dlg_responce_signal(IrrecoCmdChainDlg *self, 
					  gint response);



/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/
/* Construction & Destruction                                                 */
/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/

enum {
	IRRECO_CMD_ADD = 1,
	IRRECO_CMD_REMOVE,
	IRRECO_CMD_UP,
	IRRECO_CMD_DOWN,
	IRRECO_CMD_OK
};

G_DEFINE_TYPE(IrrecoCmdChainDlg, irreco_cmd_chain_dlg, IRRECO_TYPE_DLG)

static void irreco_cmd_chain_dlg_class_init(IrrecoCmdChainDlgClass *klass)
{
}

static void irreco_cmd_chain_dlg_init(IrrecoCmdChainDlg *self)
{
	IRRECO_ENTER
	self->editor = irreco_cmd_chain_editor_new(
		NULL, GTK_WINDOW(self), NULL);
	irreco_cmd_chain_editor_set_autosize(
		IRRECO_CMD_CHAIN_EDITOR(self->editor), 100, 600, 100, 250);
	gtk_container_add(GTK_CONTAINER(GTK_DIALOG(self)->vbox), 
			  irreco_gtk_pad(self->editor, 8, 8, 8, 8));
	
	gtk_dialog_add_buttons(GTK_DIALOG(self),
			       GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
			       GTK_STOCK_OK, GTK_RESPONSE_OK,
			       NULL);
	gtk_dialog_set_has_separator(GTK_DIALOG(self), FALSE);
	gtk_widget_show_all(GTK_WIDGET(self));
	
	g_signal_connect(G_OBJECT(self), "response",
			 G_CALLBACK(irreco_cmd_chain_dlg_responce_signal), 
			 NULL);
	
	IRRECO_RETURN
}

GtkWidget *irreco_cmd_chain_dlg_new(const gchar *title,
				    GtkWindow *parent,
				    IrrecoCmdChain *cmd_chain, 
				    IrrecoData *irreco_data)
{
	IrrecoCmdChainDlg *self;
	IRRECO_ENTER
	
	self = IRRECO_CMD_CHAIN_DLG(g_object_new(IRRECO_TYPE_CMD_CHAIN_DLG, 
				    NULL));
	irreco_gtk_dialog_set(GTK_DIALOG(self), title, parent, 
			      GTK_DIALOG_MODAL & GTK_DIALOG_DESTROY_WITH_PARENT 
			      & GTK_DIALOG_NO_SEPARATOR);
	irreco_cmd_chain_editor_set_chain(
		IRRECO_CMD_CHAIN_EDITOR(self->editor), cmd_chain);
	irreco_cmd_chain_editor_set_irreco_data(
		IRRECO_CMD_CHAIN_EDITOR(self->editor), irreco_data);
	IRRECO_RETURN_PTR(GTK_WIDGET(self));
}



/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/
/* Functions                                                                  */
/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/



/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/
/* Signal handlers                                                            */
/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/

void irreco_cmd_chain_dlg_responce_signal(IrrecoCmdChainDlg *self, 
					  gint response)
{
	IRRECO_ENTER
	if (response == GTK_RESPONSE_OK) {
		irreco_cmd_chain_editor_apply_changes(
			IRRECO_CMD_CHAIN_EDITOR(self->editor));
	} 
	IRRECO_RETURN
}












