/*
 * irreco - Ir Remote Control
 * Copyright (C) 2007,2008  Arto Karppinen (arto.karppinen@iki.fi)
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */


/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/
/* Typedef                                                                    */
/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/
/*
 * Make sure that typedefs are available before we include anything elese.
 *
 * This makes sure that whatever other structures that depend on structures 
 * defined in this file will compile OK recardles of header inclusion order.
 */
#ifndef __IRRECO_WEBDB_CACHE_H_TYPEDEF__
#define __IRRECO_WEBDB_CACHE_H_TYPEDEF__
typedef struct _IrrecoWebdbCache IrrecoWebdbCache;
#endif /* __IRRECO_WEBDB_CACHE_H_TYPEDEF__ */
 



/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/
/* Include                                                                    */
/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/
#ifndef __IRRECO_WEBDB_CACHE_H__
#define __IRRECO_WEBDB_CACHE_H__
#include "irreco_webdb.h"
#include <irreco_retry_loop.h>
#include "irreco_webdb_conf.h"

/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/
/* Datatypes                                                                  */
/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/

struct _IrrecoWebdbCache
{
	gpointer		 private;
	gboolean		 test_ok;
	IrrecoRetryLoop		*loop;
	IrrecoStringTable	*categories;
	IrrecoStringTable	*manufacturers;
	GString			*error_msg;
	GHashTable		*conf_hash;
};


/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/
/* Macro                                                                      */
/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/



/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/
/* Prototypes                                                                 */
/*=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-*/
IrrecoWebdbCache *irreco_webdb_cache_new();
void irreco_webdb_cache_free(IrrecoWebdbCache *self);
const gchar *irreco_webdb_cache_get_error(IrrecoWebdbCache *self);
gboolean irreco_webdb_cache_add_user(IrrecoWebdbCache *self, 
				     const gchar *name,
				     const gchar *email,
				     const gchar *passwd);

gboolean irreco_webdb_cache_upload_configuration(IrrecoWebdbCache *self,
						 const gchar *backend,
						 const gchar *category,
						 const gchar *file_hash,
						 const gchar *file_name,
						 const gchar *manufacturer,
						 const gchar *model,
						 const gchar *password,
						 const gchar *user,
						 const gchar *data);

gboolean irreco_webdb_cache_get_categories(IrrecoWebdbCache *self, 
					   IrrecoStringTable **categories);

gboolean irreco_webdb_cache_get_all_categories(IrrecoWebdbCache *self, 
					   IrrecoStringTable **categories);

gboolean irreco_webdb_cache_get_manufacturers(IrrecoWebdbCache *self,
					      const gchar *category, 
					      IrrecoStringTable **manufacturers);					      

gboolean irreco_webdb_cache_get_all_manufacturers(IrrecoWebdbCache *self,
					      IrrecoStringTable **manufacturers);

gboolean irreco_webdb_cache_get_models(IrrecoWebdbCache *self,
				       const gchar *category,
				       const gchar *manufacturer,
				       IrrecoStringTable **models);
				       
gboolean irreco_webdb_cache_get_configs(IrrecoWebdbCache *self, 
					const gchar *category,
					const gchar *manufacturer,
					const gchar *model,
					IrrecoStringTable **configs);
					
gboolean irreco_webdb_cache_get_configuration(IrrecoWebdbCache *self, 
				              gint id,
					      IrrecoWebdbConf **config);

gboolean irreco_webdb_cache_get_file(IrrecoWebdbCache *self, 
				     const gchar *file_hash,
	 			     const gchar *file_name,
				     GString **file_data);

gboolean irreco_webdb_cache_get_user_exists(IrrecoWebdbCache *self,
					    const gchar *name, 
	 				    gboolean *user_exists);

#endif /* __IRRECO_WEBDB_CACHE_H__ */


